@@ -172,15 +172,13 @@ nano <- function(protocol = c("bus", "pair", "push", "pull", "pub", "sub",
 
   nano[["recv"]] <- function(mode = c("serial", "character", "complex", "double",
                                       "integer", "logical", "numeric", "raw"),
-                             block = NULL,
-                             keep.raw = FALSE)
-    recv(socket, mode = mode, block = block, keep.raw = keep.raw)
+                             block = NULL)
+    recv(socket, mode = mode, block = block)
 
   nano[["recv_aio"]] <- function(mode = c("serial", "character", "complex", "double",
                                           "integer", "logical", "numeric", "raw"),
-                                 timeout = NULL,
-                                 keep.raw = FALSE)
-    recv_aio(socket, mode = mode, timeout = timeout, keep.raw = keep.raw)
+                                 timeout = NULL)
+    recv_aio(socket, mode = mode, timeout = timeout)
 
   nano[["send"]] <- function(data, mode = c("serial", "raw"), block = NULL)
     send(socket, data = data, mode = mode, block = block)
@@ -268,11 +266,9 @@ print.nanoObject <- function(x, ...) {
       attr(.subset2(x, "socket"), "id"), attr(.subset2(x, "socket"), "state"),
       attr(.subset2(x, "socket"), "protocol")), file = stdout())
   if (length(.subset2(x, "listener")))
-    cat(" - listener:", unlist(lapply(.subset2(x, "listener"), attr, "url")),
-        sep = "\n    ", file = stdout())
+    cat(" - listener:", as.character(lapply(.subset2(x, "listener"), attr, "url")), sep = "\n    ", file = stdout())
   if (length(.subset2(x, "dialer")))
-    cat(" - dialer:", unlist(lapply(.subset2(x, "dialer"), attr, "url")),
-        sep = "\n    ", file = stdout())
+    cat(" - dialer:", as.character(lapply(.subset2(x, "dialer"), attr, "url")), sep = "\n    ", file = stdout())
   invisible(x)
 
 }
@@ -284,11 +280,9 @@ print.nanoSocket <- function(x, ...) {
   cat(sprintf("< nanoSocket >\n - id: %d\n - state: %s\n - protocol: %s\n",
               attr(x, "id"), attr(x, "state"), attr(x, "protocol")), file = stdout())
   if (length(attr(x, "listener")))
-    cat(" - listener:", unlist(lapply(attr(x, "listener"), attr, "url")),
-        sep = "\n    ", file = stdout())
+    cat(" - listener:", as.character(lapply(attr(x, "listener"), attr, "url")), sep = "\n    ", file = stdout())
   if (length(attr(x, "dialer")))
-    cat(" - dialer:", unlist(lapply(attr(x, "dialer"), attr, "url")),
-        sep = "\n    ", file = stdout())
+    cat(" - dialer:", as.character(lapply(attr(x, "dialer"), attr, "url")), sep = "\n    ", file = stdout())
   invisible(x)
 
 }
@@ -298,8 +292,7 @@ print.nanoSocket <- function(x, ...) {
 print.nanoContext <- function(x, ...) {
 
   cat(sprintf("< nanoContext >\n - id: %d\n - socket: %d\n - state: %s\n - protocol: %s\n",
-              attr(x, "id"), attr(x, "socket"), attr(x, "state"), attr(x, "protocol")),
-      file = stdout())
+              attr(x, "id"), attr(x, "socket"), attr(x, "state"), attr(x, "protocol")), file = stdout())
   invisible(x)
 
 }
@@ -309,8 +302,7 @@ print.nanoContext <- function(x, ...) {
 print.nanoDialer <- function(x, ...) {
 
   cat(sprintf("< nanoDialer >\n - id: %d\n - socket: %d\n - state: %s\n - url: %s\n",
-              attr(x, "id"), attr(x, "socket"), attr(x, "state"), attr(x, "url")),
-      file = stdout())
+              attr(x, "id"), attr(x, "socket"), attr(x, "state"), attr(x, "url")), file = stdout())
   invisible(x)
 
 }
@@ -320,8 +312,7 @@ print.nanoDialer <- function(x, ...) {
 print.nanoListener <- function(x, ...) {
 
   cat(sprintf("< nanoListener >\n - id: %d\n - socket: %d\n - state: %s\n - url: %s\n",
-              attr(x, "id"), attr(x, "socket"), attr(x, "state"), attr(x, "url")),
-      file = stdout())
+              attr(x, "id"), attr(x, "socket"), attr(x, "state"), attr(x, "url")), file = stdout())
   invisible(x)
 
 }
@@ -331,13 +322,10 @@ print.nanoListener <- function(x, ...) {
 print.nanoStream <- function(x, ...) {
 
   if (length(attr(x, "dialer")))
-    cat(sprintf("< nanoStream >\n - type: dialer\n - url: %s\n - textframes: %s\n",
-                attr(x, "url"), attr(x, "textframes")), file = stdout())
-  else if (length(attr(x, "listener")))
-    cat(sprintf("< nanoStream >\n - type: listener\n - url: %s\n - textframes: %s\n",
-                attr(x, "url"), attr(x, "textframes")), file = stdout())
-  else
-    cat("< nanoStream >\n - not active\n", file = stdout())
+    cat(sprintf("< nanoStream >\n - type: dialer\n - url: %s\n - textframes: %s\n", attr(x, "url"), attr(x, "textframes")), file = stdout()) else
+      if (length(attr(x, "listener")))
+        cat(sprintf("< nanoStream >\n - type: listener\n - url: %s\n - textframes: %s\n", attr(x, "url"), attr(x, "textframes")), file = stdout()) else
+          cat("< nanoStream >\n - not active\n", file = stdout())
   invisible(x)
 
 }
@@ -346,10 +334,7 @@ print.nanoStream <- function(x, ...) {
 #'
 print.recvAio <- function(x, ...) {
 
-  if (length(x) > 2L)
-    cat("< recvAio >\n - $raw for raw message\n - $data for message data\n", file = stdout())
-  else
-    cat("< recvAio >\n - $data for message data\n", file = stdout())
+  cat("< recvAio >\n - $data for message data\n", file = stdout())
   invisible(x)
 
 }
@@ -367,7 +352,7 @@ print.sendAio <- function(x, ...) {
 #'
 print.ncurlAio <- function(x, ...) {
 
-  cat("< ncurlAio >\n - $status for response status code\n - $headers for response headers\n - $raw for raw message\n - $data for message data\n", file = stdout())
+  cat("< ncurlAio >\n - $status for response status code\n - $headers for response headers\n - $data for message data\n", file = stdout())
   invisible(x)
 
 }
@@ -376,8 +361,7 @@ print.ncurlAio <- function(x, ...) {
 #'
 print.ncurlSession <- function(x, ...) {
 
-  cat(sprintf("< ncurlSession >\n - %s\n",
-              if (length(attr(x, "aio"))) "use transact() to return data" else "not active"), file = stdout())
+  cat(sprintf("< ncurlSession >\n - %s\n", if (length(attr(x, "aio"))) "use transact() to return data" else "not active"), file = stdout())
   invisible(x)
 
 }
@@ -465,7 +449,7 @@ print.tlsConfig <- function(x, ...) {
 #' @export
 #'
 .DollarNames.recvAio <- function(x, pattern = "")
-  grep(pattern, c(if (length(x) > 2L) "raw", "data"), value = TRUE, fixed = TRUE)
+  grep(pattern, "data", value = TRUE, fixed = TRUE)
 
 #' @export
 #'
@@ -475,4 +459,4 @@ print.tlsConfig <- function(x, ...) {
 #' @export
 #'
 .DollarNames.ncurlAio <- function(x, pattern = "")
-  grep(pattern, c("status", "headers", "raw", "data"), value = TRUE, fixed = TRUE)
+  grep(pattern, c("status", "headers", "data"), value = TRUE, fixed = TRUE)
