@@ -29,41 +29,11 @@ SEXP mk_error(const int xc) {
 
 }
 
-SEXP mk_error_recv(const int xc) {
-
-  const char *names[] = {"raw", "data", ""};
-  SEXP out = PROTECT(Rf_mkNamed(VECSXP, names));
-  SEXP err = Rf_ScalarInteger(xc);
-  SET_ATTRIB(err, nano_error);
-  SET_OBJECT(err, 1);
-  SET_VECTOR_ELT(out, 0, err);
-  SET_VECTOR_ELT(out, 1, err);
-  UNPROTECT(1);
-  return out;
-
-}
-
-SEXP mk_error_ncurl(const int xc) {
-
-  const char *names[] = {"status", "headers", "raw", "data", ""};
-  SEXP out = PROTECT(Rf_mkNamed(VECSXP, names));
-  SEXP err = Rf_ScalarInteger(xc);
-  SET_ATTRIB(err, nano_error);
-  SET_OBJECT(err, 1);
-  SET_VECTOR_ELT(out, 0, err);
-  SET_VECTOR_ELT(out, 1, err);
-  SET_VECTOR_ELT(out, 2, err);
-  SET_VECTOR_ELT(out, 3, err);
-  UNPROTECT(1);
-  return out;
-
-}
-
 static void nano_write_char(R_outpstream_t stream, int c) {
 
   nano_buf *buf = (nano_buf *) stream->data;
   if (buf->cur >= buf->len) {
-    buf->len = (size_t) (2 * buf->len);
+    buf->len <<= 1;
     buf->buf = R_Realloc(buf->buf, buf->len, unsigned char);
   }
 
@@ -80,7 +50,7 @@ static void nano_write_bytes(R_outpstream_t stream, void *src, int len) {
     if (req > R_XLEN_T_MAX)
       Rf_error("serialization exceeds max length of raw vector");
     do {
-      buf->len = (size_t) (2 * buf->len);
+      buf->len <<= 1;
     } while (buf->len < req);
     buf->buf = R_Realloc(buf->buf, buf->len, unsigned char);
   }
@@ -118,44 +88,54 @@ static SEXP rawOneString(unsigned char *bytes, R_xlen_t nbytes, R_xlen_t *np) {
   char *cbuf;
   SEXP res;
 
-  for (i = *np, p = bytes + (*np); i < nbytes; p++, i++)
+  for (i = *np, p = bytes + *np; i < nbytes; p++, i++)
     if (*p == '\0') break;
 
   if (i < nbytes) {
-    p = bytes + (*np);
+    p = bytes + *np;
+    res = Rf_mkCharLenCE((char *) p, (int) (i - *np), CE_NATIVE);
     *np = i + 1;
-    res = Rf_mkChar((char *) p);
   } else {
-    cbuf = R_chk_calloc(nbytes - (*np) + 1, 1);
-    memcpy(cbuf, bytes + (*np), nbytes - (*np));
-    *np = nbytes;
-    res = Rf_mkChar(cbuf);
+    cbuf = R_chk_calloc(nbytes - *np + 1, sizeof(char));
+    memcpy(cbuf, bytes + *np, nbytes - *np);
+    res = Rf_mkCharLenCE(cbuf, (int) (nbytes - *np), CE_NATIVE);
     R_Free(cbuf);
+    *np = nbytes;
   }
 
   return res;
 
 }
 
-SEXP rawToChar(unsigned char *buf, size_t sz) {
+SEXP rawToChar(unsigned char *buf, const size_t sz) {
 
+  SEXP out;
   int i, j;
-  for (i = 0, j = -1; i < sz; i++) if (buf[i]) j = i;
+  for (i = 0, j = -1; i < sz; i++) if (buf[i]) j = i; else break;
+  if (sz - i > 1) {
+    Rf_warning("data could not be converted to a character string");
+    out = Rf_allocVector(RAWSXP, sz);
+    memcpy(STDVEC_DATAPTR(out), buf, sz);
+    return out;
+  }
 
-  return NANO_STRING((const char *) buf, j + 1);
+  PROTECT(out = Rf_allocVector(STRSXP, 1));
+  SET_STRING_ELT(out, 0, Rf_mkCharLenCE((const char *) buf, j + 1, CE_NATIVE));
+
+  UNPROTECT(1);
+  return out;
 
 }
 
-nano_buf nano_serialize(SEXP object) {
-
-  nano_buf buf;
-  struct R_outpstream_st output_stream;
+void nano_serialize(nano_buf *buf, SEXP object) {
 
   NANO_ALLOC(buf, NANONEXT_INIT_BUFSIZE);
 
+  struct R_outpstream_st output_stream;
+
   R_InitOutPStream(
     &output_stream,
-    (R_pstream_data_t) &buf,
+    (R_pstream_data_t) buf,
 #ifdef WORDS_BIGENDIAN
     R_pstream_xdr_format,
 #else
@@ -170,11 +150,34 @@ nano_buf nano_serialize(SEXP object) {
 
   R_Serialize(object, &output_stream);
 
-  return buf;
+}
+
+void nano_serialize_xdr(nano_buf *buf, SEXP object) {
+
+  NANO_ALLOC(buf, NANONEXT_INIT_BUFSIZE);
+  struct R_outpstream_st output_stream;
+
+  R_InitOutPStream(
+    &output_stream,
+    (R_pstream_data_t) buf,
+    R_pstream_xdr_format,
+    NANONEXT_SERIAL_VER,
+    nano_write_char,
+    nano_write_bytes,
+    NULL,
+    R_NilValue
+  );
+
+  R_Serialize(object, &output_stream);
 
 }
 
-SEXP nano_unserialize(unsigned char *buf, size_t sz) {
+SEXP nano_unserialize(unsigned char *buf, const size_t sz) {
+
+  if (buf[0] != 66u && buf[0] != 88u) {
+    Rf_warning("received data could not be unserialized");
+    return nano_decode(buf, sz, 8);
+  }
 
   nano_buf nbuf;
   struct R_inpstream_st input_stream;
@@ -197,60 +200,53 @@ SEXP nano_unserialize(unsigned char *buf, size_t sz) {
 
 }
 
-SEXP nano_encode(SEXP object) {
-
-  size_t sz;
-  SEXP out;
+void nano_encode(nano_buf *enc, SEXP object) {
 
   switch (TYPEOF(object)) {
   case STRSXP: ;
     const char *s;
-    unsigned char *buf;
-    size_t np, outlen = 0;
-    R_xlen_t i, xlen = XLENGTH(object);
+    R_xlen_t xlen = XLENGTH(object);
+    if (xlen == 1) {
+      s = CHAR(STRING_ELT(object, 0));
+      NANO_INIT(enc, (unsigned char *) s, strlen(s) + 1);
+      break;
+    }
+    R_xlen_t i;
+    size_t slen, outlen = 0;
     for (i = 0; i < xlen; i++)
-      outlen += strlen(Rf_translateCharUTF8(STRING_ELT(object, i))) + 1;
-    PROTECT(out = Rf_allocVector(RAWSXP, outlen));
-    buf = RAW(out);
-    for (i = 0, np = 0; i < xlen; i++) {
-      s = Rf_translateCharUTF8(STRING_ELT(object, i));
-      memcpy(buf + np, s, strlen(s) + 1);
-      np += strlen(s) + 1;
+      outlen += strlen(CHAR(STRING_ELT(object, i))) + 1;
+    NANO_ALLOC(enc, outlen);
+    for (i = 0; i < xlen; i++) {
+      s = CHAR(STRING_ELT(object, i));
+      slen = strlen(s) + 1;
+      memcpy(enc->buf + enc->cur, s, slen);
+      enc->cur += slen;
     }
-    UNPROTECT(1);
     break;
   case REALSXP:
-    sz = XLENGTH(object) * sizeof(double);
-    out = Rf_allocVector(RAWSXP, sz);
-    memcpy(RAW(out), REAL(object), sz);
+    NANO_INIT(enc, (unsigned char *) DATAPTR(object), XLENGTH(object) * sizeof(double));
     break;
   case INTSXP:
-    sz = XLENGTH(object) * sizeof(int);
-    out = Rf_allocVector(RAWSXP, sz);
-    memcpy(RAW(out), INTEGER(object), sz);
-    break;
   case LGLSXP:
-    sz = XLENGTH(object) * sizeof(int);
-    out = Rf_allocVector(RAWSXP, sz);
-    memcpy(RAW(out), LOGICAL(object), sz);
+    NANO_INIT(enc, (unsigned char *) DATAPTR(object), XLENGTH(object) * sizeof(int));
     break;
   case CPLXSXP:
-    sz = XLENGTH(object) * 2 * sizeof(double);
-    out = Rf_allocVector(RAWSXP, sz);
-    memcpy(RAW(out), COMPLEX(object), sz);
+    NANO_INIT(enc, (unsigned char *) DATAPTR(object), XLENGTH(object) * 2 * sizeof(double));
     break;
   case RAWSXP:
-    out = object;
+    NANO_INIT(enc, (unsigned char *) STDVEC_DATAPTR(object), XLENGTH(object));
+    break;
+  case NILSXP:
+    NANO_INIT(enc, NULL, 0);
     break;
   case ENVSXP:
-    out = Rf_findVarInFrame(ENCLOS(object), nano_ResultSymbol);
-    if (out != R_UnboundValue) {
-      if (TYPEOF(out) != RAWSXP) {
-        PROTECT(out);
-        nano_buf buf = nano_serialize(out);
-        out = Rf_allocVector(RAWSXP, buf.cur);
-        memcpy(RAW(out), buf.buf, buf.cur);
-        NANO_FREE(buf);
+    object = Rf_findVarInFrame(ENCLOS(object), nano_ResultSymbol);
+    if (object != R_UnboundValue) {
+      if (TYPEOF(object) == RAWSXP) {
+        NANO_INIT(enc, (unsigned char *) STDVEC_DATAPTR(object), XLENGTH(object));
+      } else {
+        PROTECT(object);
+        nano_serialize(enc, object);
         UNPROTECT(1);
       }
       break;
@@ -259,221 +255,176 @@ SEXP nano_encode(SEXP object) {
     Rf_error("'data' must be an atomic vector type to send in mode 'raw'");
   }
 
-  return out;
-
 }
 
-int nano_encodes(SEXP mode) {
+Rboolean nano_encodes(SEXP mode) {
 
-  int xc;
-  if (TYPEOF(mode) == INTSXP) {
-    xc = INTEGER(mode)[0];
-  } else {
+  if (TYPEOF(mode) != INTSXP) {
     const char *mod = CHAR(STRING_ELT(mode, 0));
     size_t slen = strlen(mod);
-    const char s[] = "serial", r[] = "raw";
-
     switch (slen) {
     case 1:
     case 2:
     case 3:
-      if (!strncmp(r, mod, slen)) { xc = 2; break; }
+      if (!strncmp(mod, "raw", slen)) return FALSE;
     case 4:
     case 5:
     case 6:
-      if (!strncmp(s, mod, slen)) { xc = 1; break; }
+      if (!strncmp(mod, "serial", slen)) return TRUE;
     default:
       Rf_error("'mode' should be one of serial, raw");
-      xc = 0;
     }
   }
 
-  return xc;
+  return INTEGER(mode)[0] == 1;
 
 }
 
 int nano_matcharg(SEXP mode) {
 
-  if (TYPEOF(mode) == INTSXP)
-    return INTEGER(mode)[0];
-
-  const char *mod = CHAR(STRING_ELT(mode, 0));
-  size_t slen = strlen(mod);
-  const char s[] = "serial", ch[] = "character", co[] = "complex", d[] = "double",
-    i[] = "integer", l[] = "logical", n[] = "numeric", r[] = "raw";
-  int xc;
-
-  switch (slen) {
-  case 1:
-    if (!strcmp("c", mod))
-      Rf_error("'mode' should be one of serial, character, complex, double, integer, logical, numeric, raw");
-  case 2:
-  case 3:
-    if (!strncmp(r, mod, slen)) { xc = 8; break; }
-  case 4:
-  case 5:
-  case 6:
-    if (!strncmp(d, mod, slen)) { xc = 4; break; }
-    if (!strncmp(s, mod, slen)) { xc = 1; break; }
-  case 7:
-    if (!strncmp(i, mod, slen)) { xc = 5; break; }
-    if (!strncmp(n, mod, slen)) { xc = 7; break; }
-    if (!strncmp(l, mod, slen)) { xc = 6; break; }
-    if (!strncmp(co, mod, slen)) { xc = 3; break; }
-  case 8:
-  case 9:
-    if (!strncmp(ch, mod, slen)) { xc = 2; break; }
-  default:
-    Rf_error("'mode' should be one of serial, character, complex, double, integer, logical, numeric, raw");
-    xc = 0;
+  if (TYPEOF(mode) != INTSXP) {
+    const char *mod = CHAR(STRING_ELT(mode, 0));
+    size_t slen = strlen(mod);
+    switch (slen) {
+    case 1:
+      if (!strncmp(mod, "c", slen) || !strncmp(mod, "s", slen))
+        Rf_error("'mode' should be one of serial, character, complex, double, integer, logical, numeric, raw, string");
+    case 2:
+    case 3:
+      if (!strncmp(mod, "raw", slen)) return 8;
+    case 4:
+    case 5:
+    case 6:
+      if (!strncmp(mod, "double", slen)) return 4;
+      if (!strncmp(mod, "serial", slen)) return 1;
+      if (!strncmp(mod, "string", slen)) return 9;
+    case 7:
+      if (!strncmp(mod, "integer", slen)) return 5;
+      if (!strncmp(mod, "numeric", slen)) return 7;
+      if (!strncmp(mod, "logical", slen)) return 6;
+      if (!strncmp(mod, "complex", slen)) return 3;
+    case 8:
+    case 9:
+      if (!strncmp(mod, "character", slen)) return 2;
+    default:
+      Rf_error("'mode' should be one of serial, character, complex, double, integer, logical, numeric, raw, string");
+    }
   }
 
-  return xc;
+  return INTEGER(mode)[0];
 
 }
 
 int nano_matchargs(SEXP mode) {
 
-  if (TYPEOF(mode) == INTSXP)
-    return INTEGER(mode)[0];
-
-  const char *mod = Rf_xlength(mode) == 8 ? CHAR(STRING_ELT(mode, 1)) : CHAR(STRING_ELT(mode, 0));
-  size_t slen = strlen(mod);
-  const char ch[] = "character", co[] = "complex", d[] = "double",
-    i[] = "integer", l[] = "logical", n[] = "numeric", r[] = "raw";
-  int xc;
-
-  switch (slen) {
-  case 1:
-  case 2:
-  case 3:
-    if (!strncmp(r, mod, slen)) { xc = 8; break; }
-  case 4:
-  case 5:
-  case 6:
-    if (!strncmp(d, mod, slen)) { xc = 4; break; }
-  case 7:
-    if (!strncmp(i, mod, slen)) { xc = 5; break; }
-    if (!strncmp(n, mod, slen)) { xc = 7; break; }
-    if (!strncmp(l, mod, slen)) { xc = 6; break; }
-    if (slen == 1)
-      Rf_error("'mode' should be one of character, complex, double, integer, logical, numeric, raw");
-    if (!strncmp(co, mod, slen)) { xc = 3; break; }
-  case 8:
-  case 9:
-    if (!strncmp(ch, mod, slen)) { xc = 2; break; }
-  default:
-    Rf_error("'mode' should be one of character, complex, double, integer, logical, numeric, raw");
-    xc = 0;
+  if (TYPEOF(mode) != INTSXP) {
+    const char *mod = XLENGTH(mode) == 9 ? CHAR(STRING_ELT(mode, 1)) : CHAR(STRING_ELT(mode, 0));
+    size_t slen = strlen(mod);
+    switch (slen) {
+    case 1:
+      if (!strncmp(mod, "c", slen))
+        Rf_error("'mode' should be one of serial, character, complex, double, integer, logical, numeric, raw, string");
+    case 2:
+    case 3:
+      if (!strncmp(mod, "raw", slen)) return 8;
+    case 4:
+    case 5:
+    case 6:
+      if (!strncmp(mod, "double", slen)) return 4;
+      if (!strncmp(mod, "string", slen)) return 9;
+    case 7:
+      if (!strncmp(mod, "integer", slen)) return 5;
+      if (!strncmp(mod, "numeric", slen)) return 7;
+      if (!strncmp(mod, "logical", slen)) return 6;
+      if (!strncmp(mod, "complex", slen)) return 3;
+    case 8:
+    case 9:
+      if (!strncmp(mod, "character", slen)) return 2;
+    default:
+      Rf_error("'mode' should be one of character, complex, double, integer, logical, numeric, raw, string");
+    }
   }
 
-  return xc;
+  return INTEGER(mode)[0];
 
 }
 
-SEXP nano_decode(unsigned char *buf, size_t sz, const int mod, const int kpr) {
+SEXP nano_decode(unsigned char *buf, size_t sz, const int mod) {
 
   SEXP data;
   size_t size;
 
   switch (mod) {
-  case 1:
-    data = nano_unserialize(buf, sz);
-    break;
   case 2:
-    PROTECT(data = Rf_allocVector(STRSXP, sz));
+    size = sz / 2 + 1;
+    PROTECT(data = Rf_allocVector(STRSXP, size));
     R_xlen_t i, m, nbytes = sz, np = 0;
-    for (i = 0, m = 0; i < sz; i++) {
+    for (i = 0, m = 0; i < size; i++) {
       SEXP onechar = rawOneString(buf, nbytes, &np);
       if (onechar == R_NilValue) break;
       SET_STRING_ELT(data, i, onechar);
-      if (XLENGTH(onechar) > 0) m++;
+      if (XLENGTH(onechar) > 0) m = i;
     }
-    data = Rf_xlengthgets(data, m);
+    if (i) SETLENGTH(data, m + 1);
     UNPROTECT(1);
-    break;
+    return data;
   case 3:
     size = 2 * sizeof(double);
-    if (sz % size == 0) {
-      data = Rf_allocVector(CPLXSXP, sz / size);
-      memcpy(COMPLEX(data), buf, sz);
-    } else {
+    if (sz % size) {
       Rf_warning("received data could not be converted to complex");
       data = Rf_allocVector(RAWSXP, sz);
-      memcpy(RAW(data), buf, sz);
+    } else {
+      data = Rf_allocVector(CPLXSXP, sz / size);
     }
     break;
   case 4:
     size = sizeof(double);
-    if (sz % size == 0) {
-      data = Rf_allocVector(REALSXP, sz / size);
-      memcpy(REAL(data), buf, sz);
-    } else {
+    if (sz % size) {
       Rf_warning("received data could not be converted to double");
       data = Rf_allocVector(RAWSXP, sz);
-      memcpy(RAW(data), buf, sz);
+    } else {
+      data = Rf_allocVector(REALSXP, sz / size);
     }
     break;
   case 5:
     size = sizeof(int);
-    if (sz % size == 0) {
-      data = Rf_allocVector(INTSXP, sz / size);
-      memcpy(INTEGER(data), buf, sz);
-    } else {
+    if (sz % size) {
       Rf_warning("received data could not be converted to integer");
       data = Rf_allocVector(RAWSXP, sz);
-      memcpy(RAW(data), buf, sz);
+    } else {
+      data = Rf_allocVector(INTSXP, sz / size);
     }
     break;
   case 6:
     size = sizeof(int);
-    if (sz % size == 0) {
-      data = Rf_allocVector(LGLSXP, sz / size);
-      memcpy(LOGICAL(data), buf, sz);
-    } else {
+    if (sz % size) {
       Rf_warning("received data could not be converted to logical");
       data = Rf_allocVector(RAWSXP, sz);
-      memcpy(RAW(data), buf, sz);
+    } else {
+      data = Rf_allocVector(LGLSXP, sz / size);
     }
     break;
   case 7:
     size = sizeof(double);
-    if (sz % size == 0) {
-      data = Rf_allocVector(REALSXP, sz / size);
-      memcpy(REAL(data), buf, sz);
-    } else {
+    if (sz % size) {
       Rf_warning("received data could not be converted to numeric");
       data = Rf_allocVector(RAWSXP, sz);
-      memcpy(RAW(data), buf, sz);
+    } else {
+      data = Rf_allocVector(REALSXP, sz / size);
     }
     break;
   case 8:
     data = Rf_allocVector(RAWSXP, sz);
-    memcpy(RAW(data), buf, sz);
     break;
+  case 9:
+    data = rawToChar(buf, sz);
+    return data;
   default:
-    data = R_NilValue;
-  }
-
-  if (kpr) {
-    PROTECT(data);
-    SEXP raw, out;
-    const char *names[] = {"raw", "data", ""};
-    PROTECT(out = Rf_mkNamed(VECSXP, names));
-    if (mod == 8) {
-      SET_VECTOR_ELT(out, 0, data);
-    } else {
-      raw = Rf_allocVector(RAWSXP, sz);
-      SET_VECTOR_ELT(out, 0, raw);
-      memcpy(RAW(raw), buf, sz);
-    }
-    SET_VECTOR_ELT(out, 1, data);
-    REprintf("'keep.raw' is deprecated and will be removed in a future package version\n");
-
-    UNPROTECT(2);
-    return out;
+    data = nano_unserialize(buf, sz);
+    return data;
   }
 
+  memcpy(STDVEC_DATAPTR(data), buf, sz);
   return data;
 
 }
@@ -511,16 +462,16 @@ SEXP rnng_ctx_open(SEXP socket) {
   PROTECT(context = R_MakeExternalPtr(ctx, nano_ContextSymbol, R_NilValue));
   R_RegisterCFinalizerEx(context, context_finalizer, TRUE);
 
-  PROTECT(klass = Rf_allocVector(STRSXP, 2));
-  SET_STRING_ELT(klass, 0, NANO_CHAR("nanoContext", 11));
-  SET_STRING_ELT(klass, 1, NANO_CHAR("nano", 4));
+  klass = Rf_allocVector(STRSXP, 2);
   Rf_classgets(context, klass);
+  SET_STRING_ELT(klass, 0, Rf_mkChar("nanoContext"));
+  SET_STRING_ELT(klass, 1, Rf_mkChar("nano"));
   Rf_setAttrib(context, nano_IdSymbol, Rf_ScalarInteger((int) ctx->id));
-  Rf_setAttrib(context, nano_StateSymbol, NANO_STRING("opened", 6));
+  Rf_setAttrib(context, nano_StateSymbol, Rf_mkString("opened"));
   Rf_setAttrib(context, nano_ProtocolSymbol, Rf_getAttrib(socket, nano_ProtocolSymbol));
   Rf_setAttrib(context, nano_SocketSymbol, Rf_ScalarInteger((int) sock->id));
 
-  UNPROTECT(2);
+  UNPROTECT(1);
   return context;
 
 }
@@ -558,7 +509,7 @@ SEXP rnng_ctx_close(SEXP context) {
   if (xc)
     ERROR_RET(xc);
 
-  Rf_setAttrib(context, nano_StateSymbol, NANO_STRING("closed", 6));
+  Rf_setAttrib(context, nano_StateSymbol, Rf_mkString("closed"));
   return nano_success;
 
 }
@@ -568,14 +519,13 @@ SEXP rnng_ctx_close(SEXP context) {
 SEXP rnng_send(SEXP con, SEXP data, SEXP mode, SEXP block) {
 
   nano_buf buf;
-  int xc, mod;
+  int xc;
 
   const SEXP ptrtag = R_ExternalPtrTag(con);
   if (ptrtag == nano_SocketSymbol) {
 
     nng_socket *sock = (nng_socket *) R_ExternalPtrAddr(con);
-    mod = nano_encodes(mode);
-    if (mod == 1) buf = nano_serialize(data); else NANO_ENCODE(buf, data);
+    nano_encodes(mode) ? nano_serialize(&buf, data) : nano_encode(&buf, data);
 
     if (block == R_NilValue) {
 
@@ -583,14 +533,13 @@ SEXP rnng_send(SEXP con, SEXP data, SEXP mode, SEXP block) {
 
     } else if (TYPEOF(block) == LGLSXP) {
 
-      const int blk = LOGICAL(block)[0];
-      xc = blk ? nng_send(*sock, buf.buf, buf.cur, 0) : nng_send(*sock, buf.buf, buf.cur, NNG_FLAG_NONBLOCK);
+      xc = nng_send(*sock, buf.buf, buf.cur, (LOGICAL(block)[0] == 0) * NNG_FLAG_NONBLOCK);
 
     } else {
 
       nng_msg *msgp;
       nng_aio *aiop;
-      nng_duration dur = (nng_duration) Rf_asInteger(block);
+      const nng_duration dur = (nng_duration) Rf_asInteger(block);
 
       if ((xc = nng_msg_alloc(&msgp, 0))) {
         NANO_FREE(buf);
@@ -621,17 +570,10 @@ SEXP rnng_send(SEXP con, SEXP data, SEXP mode, SEXP block) {
     nng_ctx *ctxp = (nng_ctx *) R_ExternalPtrAddr(con);
     nng_msg *msgp;
     nng_aio *aiop;
-    nng_duration dur;
+    const nng_duration dur = block == R_NilValue ? NNG_DURATION_DEFAULT :
+      TYPEOF(block) == LGLSXP ? (LOGICAL(block)[0] == 1) * NNG_DURATION_DEFAULT : (nng_duration) Rf_asInteger(block);
 
-    if (block == R_NilValue) {
-      dur = NNG_DURATION_DEFAULT;
-    } else if (TYPEOF(block) == LGLSXP) {
-      dur = LOGICAL(block)[0] ? NNG_DURATION_DEFAULT : 0;
-    } else {
-      dur = (nng_duration) Rf_asInteger(block);
-    }
-    mod = nano_encodes(mode);
-    if (mod == 1) buf = nano_serialize(data); else NANO_ENCODE(buf, data);
+    nano_encodes(mode) ? nano_serialize(&buf, data) : nano_encode(&buf, data);
 
     if ((xc = nng_msg_alloc(&msgp, 0))) {
       NANO_FREE(buf);
@@ -648,31 +590,25 @@ SEXP rnng_send(SEXP con, SEXP data, SEXP mode, SEXP block) {
     nng_aio_set_msg(aiop, msgp);
     nng_aio_set_timeout(aiop, dur);
     nng_ctx_send(*ctxp, aiop);
+    NANO_FREE(buf);
     nng_aio_wait(aiop);
     if ((xc = nng_aio_result(aiop)))
       nng_msg_free(nng_aio_get_msg(aiop));
     nng_aio_free(aiop);
-    NANO_FREE(buf);
 
   } else if (ptrtag == nano_StreamSymbol) {
 
     nng_stream *sp = (nng_stream *) R_ExternalPtrAddr(con);
     nng_aio *aiop;
     nng_iov iov;
-    nng_duration dur;
+    const nng_duration dur = block == R_NilValue ? NNG_DURATION_DEFAULT :
+      TYPEOF(block) == LGLSXP ? (LOGICAL(block)[0] == 1) * NNG_DURATION_DEFAULT : (nng_duration) Rf_asInteger(block);
 
-    if (block == R_NilValue) {
-      dur = NNG_DURATION_DEFAULT;
-    } else if (TYPEOF(block) == LGLSXP) {
-      dur = LOGICAL(block)[0] ? NNG_DURATION_DEFAULT : 0;
-    } else {
-      dur = (nng_duration) Rf_asInteger(block);
-    }
-    SEXP enc = nano_encode(data);
+    nano_encode(&buf, data);
 
     const int frames = LOGICAL(Rf_getAttrib(con, nano_TextframesSymbol))[0];
-    iov.iov_len = frames == 1 ? XLENGTH(enc) - 1 : XLENGTH(enc);
-    iov.iov_buf = RAW(enc);
+    iov.iov_len = buf.cur - (frames == 1);
+    iov.iov_buf = buf.buf;
 
     if ((xc = nng_aio_alloc(&aiop, NULL, NULL)))
       return mk_error(xc);
@@ -699,10 +635,9 @@ SEXP rnng_send(SEXP con, SEXP data, SEXP mode, SEXP block) {
 
 }
 
-SEXP rnng_recv(SEXP con, SEXP mode, SEXP block, SEXP keep, SEXP bytes) {
+SEXP rnng_recv(SEXP con, SEXP mode, SEXP block, SEXP bytes) {
 
   int xc;
-  const int kpr = LOGICAL(keep)[0];
   unsigned char *buf;
   size_t sz;
   SEXP res;
@@ -717,19 +652,18 @@ SEXP rnng_recv(SEXP con, SEXP mode, SEXP block, SEXP keep, SEXP bytes) {
 
       xc = nng_recv(*sock, &buf, &sz, NNG_FLAG_ALLOC + NNG_FLAG_NONBLOCK);
       if (xc)
-        return kpr ? mk_error_recv(xc) : mk_error(xc);
+        return mk_error(xc);
 
-      res = nano_decode(buf, sz, mod, kpr);
+      res = nano_decode(buf, sz, mod);
       nng_free(buf, sz);
 
     } else if (TYPEOF(block) == LGLSXP) {
 
-      const int blk = LOGICAL(block)[0];
-      xc = blk ? nng_recv(*sock, &buf, &sz, NNG_FLAG_ALLOC): nng_recv(*sock, &buf, &sz, NNG_FLAG_ALLOC + NNG_FLAG_NONBLOCK);
+      xc = nng_recv(*sock, &buf, &sz, NNG_FLAG_ALLOC + (LOGICAL(block)[0] == 0) * NNG_FLAG_NONBLOCK);
       if (xc)
-        return kpr ? mk_error_recv(xc) : mk_error(xc);
+        return mk_error(xc);
 
-      res = nano_decode(buf, sz, mod, kpr);
+      res = nano_decode(buf, sz, mod);
       nng_free(buf, sz);
 
     } else {
@@ -737,19 +671,19 @@ SEXP rnng_recv(SEXP con, SEXP mode, SEXP block, SEXP keep, SEXP bytes) {
       nng_aio *aiop;
       nng_duration dur = (nng_duration) Rf_asInteger(block);
       if ((xc = nng_aio_alloc(&aiop, NULL, NULL)))
-        return kpr ? mk_error_recv(xc) : mk_error(xc);
+        return mk_error(xc);
       nng_aio_set_timeout(aiop, dur);
       nng_recv_aio(*sock, aiop);
       nng_aio_wait(aiop);
       if ((xc = nng_aio_result(aiop))) {
         nng_aio_free(aiop);
-        return kpr ? mk_error_recv(xc) : mk_error(xc);
+        return mk_error(xc);
       }
       nng_msg *msgp = nng_aio_get_msg(aiop);
       nng_aio_free(aiop);
       buf = nng_msg_body(msgp);
       sz = nng_msg_len(msgp);
-      res = nano_decode(buf, sz, mod, kpr);
+      res = nano_decode(buf, sz, mod);
       nng_msg_free(msgp);
     }
 
@@ -757,33 +691,26 @@ SEXP rnng_recv(SEXP con, SEXP mode, SEXP block, SEXP keep, SEXP bytes) {
 
     nng_ctx *ctxp = (nng_ctx *) R_ExternalPtrAddr(con);
     const int mod = nano_matcharg(mode);
+    const nng_duration dur = block == R_NilValue ? NNG_DURATION_DEFAULT :
+      TYPEOF(block) == LGLSXP ? (LOGICAL(block)[0] == 1) * NNG_DURATION_DEFAULT : (nng_duration) Rf_asInteger(block);
     nng_aio *aiop;
-    nng_duration dur;
-
-    if (block == R_NilValue) {
-      dur = NNG_DURATION_DEFAULT;
-    } else if (TYPEOF(block) == LGLSXP) {
-      dur = LOGICAL(block)[0] ? NNG_DURATION_DEFAULT : 0;
-    } else {
-      dur = (nng_duration) Rf_asInteger(block);
-    }
 
     if ((xc = nng_aio_alloc(&aiop, NULL, NULL)))
-      return kpr ? mk_error_recv(xc) : mk_error(xc);
+      return mk_error(xc);
     nng_aio_set_timeout(aiop, dur);
     nng_ctx_recv(*ctxp, aiop);
 
     nng_aio_wait(aiop);
     if ((xc = nng_aio_result(aiop))) {
       nng_aio_free(aiop);
-      return kpr ? mk_error_recv(xc) : mk_error(xc);
+      return mk_error(xc);
     }
 
     nng_msg *msgp = nng_aio_get_msg(aiop);
     nng_aio_free(aiop);
     buf = nng_msg_body(msgp);
     sz = nng_msg_len(msgp);
-    res = nano_decode(buf, sz, mod, kpr);
+    res = nano_decode(buf, sz, mod);
     nng_msg_free(msgp);
 
   } else if (ptrtag == nano_StreamSymbol) {
@@ -791,31 +718,24 @@ SEXP rnng_recv(SEXP con, SEXP mode, SEXP block, SEXP keep, SEXP bytes) {
     nng_stream *sp = (nng_stream *) R_ExternalPtrAddr(con);
     const int mod = nano_matchargs(mode);
     const size_t xlen = (size_t) Rf_asInteger(bytes);
-    nng_duration dur;
+    const nng_duration dur = block == R_NilValue ? NNG_DURATION_DEFAULT :
+      TYPEOF(block) == LGLSXP ? (LOGICAL(block)[0] == 1) * NNG_DURATION_DEFAULT : (nng_duration) Rf_asInteger(block);
     nng_iov iov;
     nng_aio *aiop;
 
-    if (block == R_NilValue) {
-      dur = NNG_DURATION_DEFAULT;
-    } else if (TYPEOF(block) == LGLSXP) {
-      dur = LOGICAL(block)[0] ? NNG_DURATION_DEFAULT : 0;
-    } else {
-      dur = (nng_duration) Rf_asInteger(block);
-    }
-
     buf = R_Calloc(xlen, unsigned char);
     iov.iov_len = xlen;
     iov.iov_buf = buf;
 
     if ((xc = nng_aio_alloc(&aiop, NULL, NULL))) {
       R_Free(buf);
-      return kpr ? mk_error_recv(xc) : mk_error(xc);
+      return mk_error(xc);
     }
 
     if ((xc = nng_aio_set_iov(aiop, 1u, &iov))) {
       nng_aio_free(aiop);
       R_Free(buf);
-      return kpr ? mk_error_recv(xc) : mk_error(xc);
+      return mk_error(xc);
     }
 
     nng_aio_set_timeout(aiop, dur);
@@ -825,12 +745,12 @@ SEXP rnng_recv(SEXP con, SEXP mode, SEXP block, SEXP keep, SEXP bytes) {
     if ((xc = nng_aio_result(aiop))) {
       nng_aio_free(aiop);
       R_Free(buf);
-      return kpr ? mk_error_recv(xc) : mk_error(xc);
+      return mk_error(xc);
     }
 
     sz = nng_aio_count(aiop);
     nng_aio_free(aiop);
-    res = nano_decode(buf, sz, mod, kpr);
+    res = nano_decode(buf, sz, mod);
     R_Free(buf);
 
   } else {
@@ -1010,29 +930,21 @@ SEXP rnng_subscribe(SEXP object, SEXP value, SEXP sub) {
     Rf_error("'object' is not a valid Socket, Context, Stream, Listener or Dialer");
 
   const char *op = LOGICAL(sub)[0] ? "sub:subscribe" : "sub:unsubscribe";
+  nano_buf buf;
   int xc;
 
   const SEXP ptrtag = R_ExternalPtrTag(object);
   if (ptrtag == nano_SocketSymbol) {
 
     nng_socket *sock = (nng_socket *) R_ExternalPtrAddr(object);
-    if (value == R_NilValue) {
-        xc = nng_socket_set(*sock, op, NULL, 0);
-      } else {
-        SEXP enc = nano_encode(value);
-        size_t sz = TYPEOF(value) == STRSXP ? (size_t) XLENGTH(enc) - 1 : (size_t) XLENGTH(enc);
-        xc = nng_socket_set(*sock, op, RAW(enc), sz);
-      }
+    nano_encode(&buf, value);
+    xc = nng_socket_set(*sock, op, buf.buf, buf.cur - (TYPEOF(value) == STRSXP));
 
   } else if (ptrtag == nano_ContextSymbol) {
 
     nng_ctx *ctx = (nng_ctx *) R_ExternalPtrAddr(object);
-    if (value == R_NilValue) {
-      xc = nng_ctx_set(*ctx, op, NULL, 0);
-    } else {
-      SEXP enc = nano_encode(value);
-      xc = nng_ctx_set(*ctx, op, RAW(enc), XLENGTH(enc));
-    }
+    nano_encode(&buf, value);
+    xc = nng_ctx_set(*ctx, op, buf.buf, buf.cur - (TYPEOF(value) == STRSXP));
 
   } else {
     Rf_error("'object' is not a valid Socket or Context");
@@ -1053,29 +965,31 @@ SEXP rnng_get_opt(SEXP object, SEXP opt) {
   const char *op = CHAR(STRING_ELT(opt, 0));
   SEXP out;
   int xc, typ;
-  bool bval;
-  int ival;
-  nng_duration dval;
-  size_t sval;
-  char *strval;
-  uint64_t uval;
+  union optval_u {
+    char *str;
+    bool b;
+    nng_duration d;
+    int i;
+    size_t s;
+    uint64_t u;
+  } optval;
 
   const SEXP ptrtag = R_ExternalPtrTag(object);
   if (ptrtag == nano_SocketSymbol) {
 
     nng_socket *sock = (nng_socket *) R_ExternalPtrAddr(object);
     for (;;) {
-      xc = nng_socket_get_string(*sock, op, &strval);
+      xc = nng_socket_get_string(*sock, op, &optval.str);
       if (xc == 0) { typ = 1; break; }
-      xc = nng_socket_get_ms(*sock, op, &dval);
+      xc = nng_socket_get_ms(*sock, op, &optval.d);
       if (xc == 0) { typ = 2; break; }
-      xc = nng_socket_get_size(*sock, op, &sval);
+      xc = nng_socket_get_size(*sock, op, &optval.s);
       if (xc == 0) { typ = 3; break; }
-      xc = nng_socket_get_int(*sock, op, &ival);
+      xc = nng_socket_get_int(*sock, op, &optval.i);
       if (xc == 0) { typ = 4; break; }
-      xc = nng_socket_get_bool(*sock, op, &bval);
+      xc = nng_socket_get_bool(*sock, op, &optval.b);
       if (xc == 0) { typ = 5; break; }
-      xc = nng_socket_get_uint64(*sock, op, &uval);
+      xc = nng_socket_get_uint64(*sock, op, &optval.u);
       typ = 6; break;
     }
 
@@ -1083,17 +997,17 @@ SEXP rnng_get_opt(SEXP object, SEXP opt) {
 
     nng_ctx *ctx = (nng_ctx *) R_ExternalPtrAddr(object);
     for (;;) {
-      xc = nng_ctx_get_string(*ctx, op, &strval);
+      xc = nng_ctx_get_string(*ctx, op, &optval.str);
       if (xc == 0) { typ = 1; break; }
-      xc = nng_ctx_get_ms(*ctx, op, &dval);
+      xc = nng_ctx_get_ms(*ctx, op, &optval.d);
       if (xc == 0) { typ = 2; break; }
-      xc = nng_ctx_get_size(*ctx, op, &sval);
+      xc = nng_ctx_get_size(*ctx, op, &optval.s);
       if (xc == 0) { typ = 3; break; }
-      xc = nng_ctx_get_int(*ctx, op, &ival);
+      xc = nng_ctx_get_int(*ctx, op, &optval.i);
       if (xc == 0) { typ = 4; break; }
-      xc = nng_ctx_get_bool(*ctx, op, &bval);
+      xc = nng_ctx_get_bool(*ctx, op, &optval.b);
       if (xc == 0) { typ = 5; break; }
-      xc = nng_ctx_get_uint64(*ctx, op, &uval);
+      xc = nng_ctx_get_uint64(*ctx, op, &optval.u);
       typ = 6; break;
     }
 
@@ -1101,17 +1015,17 @@ SEXP rnng_get_opt(SEXP object, SEXP opt) {
 
     nng_stream *st = (nng_stream *) R_ExternalPtrAddr(object);
     for (;;) {
-      xc = nng_stream_get_string(st, op, &strval);
+      xc = nng_stream_get_string(st, op, &optval.str);
       if (xc == 0) { typ = 1; break; }
-      xc = nng_stream_get_ms(st, op, &dval);
+      xc = nng_stream_get_ms(st, op, &optval.d);
       if (xc == 0) { typ = 2; break; }
-      xc = nng_stream_get_size(st, op, &sval);
+      xc = nng_stream_get_size(st, op, &optval.s);
       if (xc == 0) { typ = 3; break; }
-      xc = nng_stream_get_int(st, op, &ival);
+      xc = nng_stream_get_int(st, op, &optval.i);
       if (xc == 0) { typ = 4; break; }
-      xc = nng_stream_get_bool(st, op, &bval);
+      xc = nng_stream_get_bool(st, op, &optval.b);
       if (xc == 0) { typ = 5; break; }
-      xc = nng_stream_get_uint64(st, op, &uval);
+      xc = nng_stream_get_uint64(st, op, &optval.u);
       typ = 6; break;
     }
 
@@ -1119,17 +1033,17 @@ SEXP rnng_get_opt(SEXP object, SEXP opt) {
 
     nng_listener *list = (nng_listener *) R_ExternalPtrAddr(object);
     for (;;) {
-      xc = nng_listener_get_string(*list, op, &strval);
+      xc = nng_listener_get_string(*list, op, &optval.str);
       if (xc == 0) { typ = 1; break; }
-      xc = nng_listener_get_ms(*list, op, &dval);
+      xc = nng_listener_get_ms(*list, op, &optval.d);
       if (xc == 0) { typ = 2; break; }
-      xc = nng_listener_get_size(*list, op, &sval);
+      xc = nng_listener_get_size(*list, op, &optval.s);
       if (xc == 0) { typ = 3; break; }
-      xc = nng_listener_get_int(*list, op, &ival);
+      xc = nng_listener_get_int(*list, op, &optval.i);
       if (xc == 0) { typ = 4; break; }
-      xc = nng_listener_get_bool(*list, op, &bval);
+      xc = nng_listener_get_bool(*list, op, &optval.b);
       if (xc == 0) { typ = 5; break; }
-      xc = nng_listener_get_uint64(*list, op, &uval);
+      xc = nng_listener_get_uint64(*list, op, &optval.u);
       typ = 6; break;
     }
 
@@ -1137,17 +1051,17 @@ SEXP rnng_get_opt(SEXP object, SEXP opt) {
 
     nng_dialer *dial = (nng_dialer *) R_ExternalPtrAddr(object);
     for (;;) {
-      xc = nng_dialer_get_string(*dial, op, &strval);
+      xc = nng_dialer_get_string(*dial, op, &optval.str);
       if (xc == 0) { typ = 1; break; }
-      xc = nng_dialer_get_ms(*dial, op, &dval);
+      xc = nng_dialer_get_ms(*dial, op, &optval.d);
       if (xc == 0) { typ = 2; break; }
-      xc = nng_dialer_get_size(*dial, op, &sval);
+      xc = nng_dialer_get_size(*dial, op, &optval.s);
       if (xc == 0) { typ = 3; break; }
-      xc = nng_dialer_get_int(*dial, op, &ival);
+      xc = nng_dialer_get_int(*dial, op, &optval.i);
       if (xc == 0) { typ = 4; break; }
-      xc = nng_dialer_get_bool(*dial, op, &bval);
+      xc = nng_dialer_get_bool(*dial, op, &optval.b);
       if (xc == 0) { typ = 5; break; }
-      xc = nng_dialer_get_uint64(*dial, op, &uval);
+      xc = nng_dialer_get_uint64(*dial, op, &optval.u);
       typ = 6; break;
     }
 
@@ -1160,22 +1074,22 @@ SEXP rnng_get_opt(SEXP object, SEXP opt) {
 
   switch (typ) {
   case 1:
-    out = Rf_mkString(strval);
+    out = Rf_mkString(optval.str);
     break;
   case 2:
-    out = Rf_ScalarInteger((int) dval);
+    out = Rf_ScalarInteger((int) optval.d);
     break;
   case 3:
-    out = Rf_ScalarInteger((int) sval);
+    out = Rf_ScalarInteger((int) optval.s);
     break;
   case 4:
-    out = Rf_ScalarInteger(ival);
+    out = Rf_ScalarInteger(optval.i);
     break;
   case 5:
-    out = Rf_ScalarLogical((int) bval);
+    out = Rf_ScalarInteger((int) optval.b);
     break;
   default:
-    out = Rf_ScalarReal((double) uval);
+    out = Rf_ScalarReal((double) optval.u);
   }
 
   return out;
@@ -1223,26 +1137,13 @@ SEXP rnng_stats_get(SEXP object, SEXP stat) {
   }
 
   typ = nng_stat_type(sst);
-  if (typ == NNG_STAT_STRING) {
-    out = Rf_mkString(nng_stat_string(sst));
-  } else {
-    out = Rf_ScalarReal((double) nng_stat_value(sst));
-  }
+  out = typ == NNG_STAT_STRING ? Rf_mkString(nng_stat_string(sst)) : Rf_ScalarReal((double) nng_stat_value(sst));
 
   nng_stats_free(nst);
   return out;
 
 }
 
-// terminate the library -------------------------------------------------------
-
-SEXP rnng_fini(void) {
-
-  nng_fini();
-  return R_NilValue;
-
-}
-
 // weakref ---------------------------------------------------------------------
 
 SEXP rnng_weakref_make(SEXP key, SEXP value) {
@@ -1265,15 +1166,20 @@ SEXP rnng_weakref_value(SEXP w) {
 
 SEXP rnng_strcat(SEXP a, SEXP b) {
 
+  SEXP out;
   const char *ap = CHAR(STRING_ELT(a, 0));
   const char *bp = CHAR(STRING_ELT(b, 0));
-  R_xlen_t alen = XLENGTH(STRING_ELT(a, 0));
-  R_xlen_t blen = XLENGTH(STRING_ELT(b, 0));
+  const size_t alen = strlen(ap);
+  const size_t blen = strlen(bp);
 
   char *buf = R_alloc(sizeof(char), alen + blen + 1);
   memcpy(buf, ap, alen);
   memcpy(buf + alen, bp, blen + 1);
 
-  return NANO_STRING(buf, alen + blen);
+  PROTECT(out = Rf_allocVector(STRSXP, 1));
+  SET_STRING_ELT(out, 0, Rf_mkCharLenCE(buf, alen + blen, CE_NATIVE));
+
+  UNPROTECT(1);
+  return out;
 
 }
