@@ -70,18 +70,17 @@ nanotestp(raio)
 nanotest(is_error_value(call_aio(raio)$data))
 nanotest(is_error_value(raio$data))
 nanotestz(n$send(data.frame(), block = FALSE))
-nanotest(is.list(n1$recv(block = FALSE, keep.raw = TRUE)))
-nanotestz(n1$send("test", mode = "raw", block = 500))
-nanotest(n$recv("character", block = 500, keep.raw = TRUE)$data == "test")
+nanotestnn(n1$recv(block = FALSE))
+nanotestz(n1$send(c("test", "", "spec"), mode = "raw", block = 500))
+nanotest(identical(n$recv("character", block = 500), c("test", "", "spec")))
 nanotestz(n1$send(c(1L, 0L, 0L), mode = "r"))
-nanotest(is.list(n$recv("int", keep.raw = TRUE)))
+nanotest(is.integer(n$recv("int")))
 nanotestaio(saio <- n$send_aio(data.frame(random(1e4L)), timeout = 900))
 nanotestp(saio)
 nanotestaio(call_aio(saio))
-nanotest(!is_nano(saio))
 nanotest(is.integer(saio$result))
 nanotesterr(n$send("wrong mode", mode = "none"), "mode")
-nanotestaio(raio <- n1$recv_aio(timeout = 500, keep.raw = TRUE))
+nanotestaio(raio <- n1$recv_aio(timeout = 500))
 nanotestp(raio)
 raio$newfield <- "doesnotwork"
 raio[["newfield"]] <- "doesnotwork"
@@ -92,22 +91,21 @@ saio[["newfield"]] <- "doesnotwork"
 nanotestn(saio$newfield)
 nanotest(is.logical(.unresolved(saio)))
 nanotest(is.logical(unresolved(saio)))
-nanotestnn(raio$raw)
-nanotest(identical(unserialize(call_aio(raio)$raw), raio$data))
-nanotestaio(msg <- n1$recv_aio(mode = "numer", keep.raw = TRUE, timeout = 500))
+nanotestaio(msg <- n1$recv_aio(mode = "numer", timeout = 500))
 nanotest(identical(call_aio(msg), msg))
-nanotestaio(msg <- n1$recv_aio(mode = "complex", keep.raw = TRUE, timeout = 500))
+nanotestaio(msg <- n1$recv_aio(mode = "complex", timeout = 500))
 nanotestn(stop_aio(msg))
 nanotestn(stop_aio(n))
 nanotest(identical(call_aio(msg), msg))
-nanotesterr(msg$raw, "active")
 nanotesterr(msg$data, "active")
 nanotest(identical(call_aio(n), n))
-nanotest(!is_aio(n))
 nanotestaio(sraio <- n$send_aio(as.raw(0L), mode = "r", timeout = 500))
-nanotestaio(rraio <- n1$recv_aio(mode = "r", timeout = 500))
+nanotestaio(rraio <- n1$recv_aio(mode = 1L, timeout = 500))
+nanotestw(is_nul_byte(call_aio(rraio)$data))
+nanotestaio(sraio <- n$send_aio(as.raw(1L), mode = "ra", timeout = 500))
+nanotestaio(rraio <- n1$recv_aio(mode = "raw", timeout = 500))
 nanotest(is.raw(call_aio(rraio)$data))
-nanotestaio(sraio <- n$send_aio(c(1+2i, 4+3i), mode = "ra", timeout = 500))
+nanotestaio(sraio <- n$send_aio(c(1+2i, 4+3i), mode = "raw", timeout = 500))
 nanotestaio(rraio <- n1$recv_aio(mode = "co", timeout = 500))
 nanotest(is.complex(call_aio(rraio)$data))
 nanotestaio(sraio <- n$send_aio(5, mode = "raw", timeout = 500))
@@ -131,16 +129,18 @@ nanotestw(is.raw(call_aio(rraio)$data))
 nanotestaio(sraio <- n$send_aio(as.raw(0L), mode = "raw", timeout = 500))
 nanotestaio(rraio <- n1$recv_aio(mode = "numeric", timeout = 500))
 nanotestw(is.raw(call_aio(rraio)$data))
+nanotestaio(sraio <- n$send_aio(as.raw(0L), mode = "raw", timeout = 500))
+nanotestaio(rraio <- n1$recv_aio(mode = "complex", timeout = 500))
+nanotestw(is.raw(call_aio(rraio)$data))
 nanotestaio(sraio <- n$send_aio("hello", mode = 2L, timeout = 500))
 assign("aio", value = "test", envir = sraio)
 nanotesterr(sraio$result, "valid")
-nanotestaio(rraio <- n1$recv_aio(mode = "characte", keep.raw = TRUE, timeout = 500))
+nanotestaio(rraio <- n1$recv_aio(mode = "charac", timeout = 500))
 nanotesterr(opt(rraio[["aio"]], "false") <- 0L, "valid")
 nanotesterr(subscribe(rraio[["aio"]], "false"), "valid")
 nanotesterr(opt(rraio[["aio"]], "false"), "valid")
 nanotesterr(stat(rraio[["aio"]], "pipes"), "valid")
 assign("aio", value = "test", envir = rraio)
-nanotesterr(rraio$raw, "valid")
 nanotesterr(rraio$data, "valid")
 
 nanotestz(n$dial(url = "inproc://two", autostart = FALSE))
@@ -194,22 +194,26 @@ nanotest(is.integer(call_aio(csaio)$result))
 nanotestaio(craio <- recv_aio(ctx, timeout = 500))
 nanotest(is.list(.subset2(call_aio(craio), "data")))
 nanotest(is.integer(req$send("context test", mode ="raw", block = 500)))
-nanotest(recv(ctx, mode = "ch", block = 500) == "context test")
+nanotest(recv(ctx, mode = "string", block = 500) == "context test")
 nanotestnn(req$send(data.frame(), mode = "seri", block = 500))
-nanotestaio(msg <- recv_aio(ctx, mode = "ser", timeout = 500, keep.raw = FALSE))
+nanotestaio(msg <- recv_aio(ctx, mode = "ser", timeout = 500))
 nanotest(is.logical(.unresolved(msg)))
 nanotest(is.logical(unresolved(msg)))
-nanotest(is.list(call_aio(msg)$data))
+nanotest(is.data.frame(call_aio(msg)$data))
 nanotest(!unresolved(msg))
 nanotest(is.integer(req$send(c(TRUE, FALSE, TRUE), mode = 2L, block = 500)))
-nanotestaio(msg <- recv_aio(ctx, mode = 6L, timeout = 500, keep.raw = TRUE))
-nanotest(is.raw(call_aio(msg)$raw))
-nanotestaio(err <- send_aio(ctx, "test"))
+nanotestaio(msg <- recv_aio(ctx, mode = 6L, timeout = 500))
+nanotest(is.logical(call_aio(msg)$data))
+nanotestaio(err <- send_aio(ctx, msg, mode = "raw"))
 nanotestn(stop_aio(err))
 nanotesterr(err$result, "active")
 nanotestaio(err <- send_aio(ctx, "test"))
 nanotest(is_error_value(call_aio(err)$result))
-nanotestaio(rek <- request(req$context, c(1+3i, 4+2i), recv_mode = "complex", timeout = 500, keep.raw = TRUE))
+nanotest(is.integer(req$send(as.raw(1L), mode = 2L, block = 500)))
+nanotestaio(msg <- call_aio(recv_aio(ctx, mode = 8L, timeout = 500)))
+nanotestaio(envraw <- send_aio(ctx, msg, mode = 2L, timeout = 500))
+nanotest(identical(req$recv(mode = 8L, block = 500), as.raw(1L)))
+nanotestaio(rek <- request(req$context, c(1+3i, 4+2i), recv_mode = "complex", timeout = 500))
 nanotest(is.integer(reply(ctx, execute = identity, send_mode = "ra", timeout = 500)))
 nanotest(is.complex(call_aio(rek)[["data"]]))
 
@@ -217,11 +221,11 @@ nanotest(inherits(cv <- cv(), "conditionVariable"))
 nanotestp(cv)
 nanotest(until(cv, 10L))
 nanotest(until(cv, 10))
-nanotestn(cv_reset(cv))
+nanotestz(cv_reset(cv))
 nanotestz(cv_value(cv))
-nanotestaio(cr <- request_signal(req$context, "test", timeout = 500, keep.raw = TRUE, cv = cv))
-nanotestaio(cr <- recv_aio_signal(rep, timeout = 500, keep.raw = TRUE, cv = cv))
-nanotest(is.raw(call_aio(cr)$raw))
+nanotestaio(cr <- request_signal(req$context, "test", timeout = 500, cv = cv))
+nanotestaio(cr <- recv_aio_signal(rep, timeout = 500, cv = cv))
+nanotest(call_aio(cr)$data == "test")
 nanotestaio(cr <- recv_aio_signal(context(rep), timeout = 500, cv = cv))
 nanotestz(pipe_notify(rep, cv))
 nanotestz(pipe_notify(req$socket, cv = cv, add = FALSE, remove = TRUE, flag = FALSE))
@@ -243,6 +247,7 @@ nanotestn(req$context)
 rep$dialer <- NULL
 nanotestxp(rep$dialer[[1L]])
 nanotestz(close(ctx))
+nanotestw(close(ctx) == 7L)
 nanotestz(close(rep))
 
 nanotestnano(pub <- nano("pub", listen = "inproc://ps"))
@@ -269,7 +274,7 @@ nanotestz(sub$close())
 nanotestz(pub$close())
 nanotest(wait(cv))
 nanotest(wait(cv2))
-nanotestn(cv_signal(cv))
+nanotestz(cv_signal(cv))
 nanotest(cv_value(cv) == 1L)
 
 nanotestnano(surv <- nano(protocol = "surveyor", listen = "inproc://sock1", dial = "inproc://sock2"))
@@ -289,7 +294,7 @@ nanotestz(surv$close())
 nanotestz(resp$close())
 nanotest(!wait(cv))
 nanotest(!wait(cv2))
-nanotest(is_error_value(resp$recv(keep.raw = TRUE)$raw))
+nanotest(is_error_value(resp$recv()))
 
 nanotest(inherits(bus <- socket(protocol = "bus"), "nanoSocket"))
 nanotest(inherits(push <- socket(protocol = "push"), "nanoSocket"))
@@ -314,30 +319,31 @@ nanotestnn(ncurl("http://www.cam.ac.uk/", follow = FALSE, response = list("date"
 nanotestnn(ncurl("http://www.cam.ac.uk/", follow = TRUE))
 nanotestnn(ncurl("http://postman-echo.com/post", convert = FALSE, method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = "server", timeout = 3000L))
 nanotestnn(ncurl("http://postman-echo.com/post", convert = FALSE, method = "POST", headers = list(`Content-Type` = "text/plain"), response = list("Date", "Server"), timeout = 3000))
-relo <- ncurl("https://trin.cam.ac.uk/", convert = FALSE, response = "server", async = TRUE)
+relo <- ncurl_aio("https://trin.cam.ac.uk/", convert = FALSE, response = "server")
 nanotest(is.integer(call_aio(relo)$status))
-relo <- ncurl("https://trin.cam.ac.uk/", convert = FALSE, response = list("server"), async = TRUE)
+relo <- ncurl_aio("https://trin.cam.ac.uk/", convert = FALSE, response = list("server"))
 nanotest(is.integer(call_aio(relo)$status))
-put1 <- ncurl("http://postman-echo.com/put", async = TRUE, method = "PUT", headers = c(Authorization = "Bearer token"), data = "test", response = c("date", "server"), timeout = 3000L)
+put1 <- ncurl_aio("http://postman-echo.com/put", method = "PUT", headers = c(Authorization = "Bearer token"), data = "test", response = c("date", "server"), timeout = 3000L)
 nanotest(is.integer(call_aio(put1)$status))
 nanotestnn(put1$headers)
-nanotest(is.null(put1$raw) || is_error_value(put1$raw))
 nanotestnn(put1$data)
-put2 <- ncurl("http://postman-echo.com/put", async = TRUE, method = "PUT", headers = list(Authorization = "Bearer token"), response = list("date", "server"), timeout = 3000L)
+put2 <- ncurl_aio("http://postman-echo.com/put", method = "PUT", headers = list(Authorization = "Bearer token"), response = list("date", "server"), timeout = 3000L)
 nanotestp(put2)
 nanotest(is.integer(call_aio(put2)$status))
 nanotest(is.null(stop_aio(put2)))
-nanotest(is_error_value(ncurl("http")$raw))
-nanotest(is_error_value(ncurl("http", async = TRUE)$data))
-haio <- ncurl("https://i.i", async = TRUE)
-nanotest(is_error_value(call_aio(haio)$raw))
+nanotest(is_error_value(ncurl("http")$data))
+nanotest(is_error_value(ncurl_aio("http")$data))
+haio <- ncurl_aio("https://i.i")
+nanotest(is_error_value(call_aio(haio)$data))
 nanotestp(haio$data)
+ncaio <- ncurl_aio("https://shikokuchuo.net/nanonext/reference/figures/logo.png")
+nanotestw(is.raw(call_aio(ncaio)$data) || is_error_value(ncaio$data))
 sess <- ncurl_session("https://postman-echo.com/post", method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = "server", timeout = 3000L)
 nanotestnn(sess)
-nanotest(is_error_value(sess) || length(transact(sess)) == 4L)
+nanotest(is_error_value(sess) || length(transact(sess)) == 3L)
 sess <- ncurl_session("https://postman-echo.com/post", convert = FALSE, method = "POST", headers = list(`Content-Type` = "text/plain"), response = list("Date", "Server"), timeout = 3000)
 nanotestnn(sess)
-nanotest(is_error_value(sess) || length(transact(sess)) == 4L)
+nanotest(is_error_value(sess) || length(transact(sess)) == 3L)
 nanotest(is_error_value(sess) || close(sess) == 0L)
 nanotesterr(transact(sess), "ncurlSession")
 nanotestw(is_error_value(ncurl_session("https://i")))
@@ -355,15 +361,13 @@ nanotest(is.double(mclock()))
 nanotestn(msleep(1L))
 nanotestn(msleep(1))
 nanotestn(msleep("a"))
-nanotest(is.character(urlp <- parse_url("tcp://192.168.0.2:5555")) && length(urlp) == 10L)
+nanotest(is.character(urlp <- parse_url("://")) && length(urlp) == 10L)
+nanotest(all(nzchar(parse_url("wss://use:r@[::1]/path?q=1#name"))))
 nanotest(is.double(random()))
 nanotest(length(random(2)) == 2L)
 nanotest(length(random(3L)) == 3L)
 nanotesterr(parse_url("tcp:/"), "argument")
 nanotesterr(random("test"), "integer")
-nanotesterr(device(push, pull), "closed")
-nanotesterr(device(ctx, push), "s1")
-nanotesterr(device(push, ctx), "s2")
 for (i in c(100:103, 200:208, 226, 300:308, 400:426, 428:431, 451, 500:511))
   nanotest(is.character(status_code(i)))
 nanotest(strcat("hello ", "world") == "hello world")
@@ -374,10 +378,10 @@ if (length(s)) {
   nanotest(is.integer(send(s, "message1", block = 500L)))
   nanotestnn(recv(s, block = FALSE))
   nanotest(is.integer(send(s, "message2", block = FALSE)))
-  nanotestnn(recv(s, mode = 2L, block = 100))
+  nanotestnn(recv(s, mode = 9L, block = 100))
   nanotest(is.integer(send(s, 2L, block = 500)))
-  nanotestaio(sr <- recv_aio(s, mode = "integer", keep.raw = TRUE, timeout = 500L, n = 8192L))
-  nanotestw(length(call_aio(sr)[["raw"]]))
+  nanotestaio(sr <- recv_aio(s, mode = "integer", timeout = 500L, n = 8192L))
+  nanotestw(length(call_aio(sr)[["data"]]))
   nanotestn(stop_aio(sr))
   nanotestaio(ss <- send_aio(s, "async", timeout = 500L))
   nanotest(is.integer(call_aio(ss)[["result"]]))
@@ -462,8 +466,8 @@ fakelist <- `class<-`("test", "nanoListener")
 nanotesterr(start(fakelist), "valid Listener")
 nanotesterr(close(fakelist), "valid Listener")
 unres <- `class<-`(NA, "unresolvedValue")
-nanotestp(unres)
 nanotest(!unresolved(unres))
+nanotestp(unres)
 nanotest(identical(call_aio("a"), "a"))
 nanotestn(stop_aio("a"))
 
@@ -485,6 +489,7 @@ nanotest(is.raw(base64enc(data.frame(), convert = FALSE)))
 nanotest(is.raw(base64dec(base64enc(as.raw(c(1L, 2L)), convert = FALSE), convert = FALSE)))
 nanotest(is.integer(base64dec(base64enc(c(1L, 2L)), convert = NA)))
 nanotesterr(base64dec("__"), "not valid base64")
+nanotesterr(base64dec(404), "not valid base64")
 
 k <- new.env()
 v <- "value"
@@ -509,7 +514,10 @@ nanotestxp(tls <- tls_config(client = cert$client))
 nanotest(inherits(tls, "tlsConfig"))
 nanotestp(tls)
 nanotest(is_error_value(ncurl("https://www.cam.ac.uk/", tls = tls)$status))
-nanotest(is_error_value(call_aio(ncurl("https://www.cam.ac.uk/", async = TRUE, tls = tls))$status))
+nanotest(is_error_value(call_aio(ncurl_aio("https://www.cam.ac.uk/", tls = tls))$status))
+nanotesterr(ncurl_session("https://www.cam.ac.uk/", tls = cert$client), "not a valid TLS")
+nanotestw(typeof(b <- ncurl_session("https://www.cam.ac.uk/", tls = tls)) == "externalptr" || is_error_value(b))
+nanotest(if (!is_error_value(b)) is_error_value(transact(b)[["headers"]]) else TRUE)
 nanotestxp(s <- socket(listen = "tls+tcp://127.0.0.1:5556", tls = tls_config(server = cert$server)))
 nanotestxp(s1 <- socket(dial = "tls+tcp://127.0.0.1:5556", tls = tls))
 nanotestw(dial(s, url = "tls+tcp://[", tls = tls, error = FALSE) > 0)
